(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: translate_spec.mli,v 1.6 2014/05/06 12:56:22 fclement Exp $ *)

(** {3 Interface to the external part of State maker functor}
 
 Translation between values of the specific types used in the specialized
 algorithm and the more general types of values that will travel on wires.
 
 Injection: from specific types to wire types.
 Projection: from wire types to specific types. *)

open Scalar_spec;;
open Components_spec;;

module type MAKER =
  functor (Scal : SCALAR) ->
  functor (Comps : COMPONENTS) ->
sig
  module Stt : module type of (State_types.Make (Scal) (Comps));;
  
  type scalar = Scal.t;;
  type 'a components = 'a Comps.t;;
  
  type gradients = Stt.gradients;;
  type partitions = Stt.partitions;;
  type state = Stt.state;;
  type selected_cutting = Stt.selected_cutting;;
  
  (** {6 Init} *)
  
  val inj_init : unit -> Wire_types.dimension;;
  val proj_coarse_init :
    Wire_types.size -> Wire_types.partition ->
    Wire_types.coarse_parameter -> Wire_types.cost -> state
  ;;
  
  (** {6 Fine} *)
  
  val make_fine :
    state ->
    (unit -> Wire_types.fine_parameter) *
    (Wire_types.fine_gradient -> gradients)
  ;;
  
  (** {6 Coarse} *)
  
  val make_coarse :
    state -> partitions ->
    (unit -> Wire_types.partition * Wire_types.coarse_parameter) *
    (Wire_types.coarse_parameter -> Wire_types.cost -> selected_cutting)
  ;;
end
;;
